#include <Adafruit_NeoPixel.h>
#include <Encoder.h>

// LED setup
#define LED_PIN     D6
#define NUM_LEDS    48
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800);

// Encoder setup
#define ENCODER_PIN_A D2
#define ENCODER_PIN_B D3
#define BUTTON_PIN D4

Encoder myEnc(ENCODER_PIN_A, ENCODER_PIN_B);

enum Mode { HUE, BRIGHTNESS };
Mode currentMode = HUE;

long oldPosition  = -999;
int hue = 0;
int brightness = 255;
long huePosition = 0;
long brightnessPosition = 255 / 4;

void setup() {
  pinMode(BUTTON_PIN, INPUT_PULLUP);

  strip.begin();
  strip.show(); // Initialize all pixels to 'off'

  Serial.begin(9600);
}

void loop() {
  // Read the encoder
  long newPosition = myEnc.read() / 4;
  if (newPosition != oldPosition) {
    oldPosition = newPosition;
    
    if (currentMode == HUE) {
      huePosition = newPosition;
      hue = (huePosition * 4) % 256; // Adjusting hue change for fewer turns
      if (hue < 0) hue += 256;
    } else {
      brightnessPosition = newPosition;
      brightness = constrain(brightnessPosition * 8, 0, 255); // Adjusting brightness change for fewer turns
      if (brightnessPosition < 0) {
        brightnessPosition = 0;
        brightness = 0;
        myEnc.write(0);
      } else if (brightnessPosition > 255 / 4) {
        brightnessPosition = 255 / 4;
        brightness = 255;
        myEnc.write(255 / 4 * 4);
      }
    }

    Serial.print("Hue: ");
    Serial.print(hue);
    Serial.print(", Brightness: ");
    Serial.println(brightness);

    updateLEDs();
  }

  // Check the button
  static bool lastButtonState = HIGH;
  bool buttonState = digitalRead(BUTTON_PIN);
  
  if (lastButtonState == HIGH && buttonState == LOW) {
    if (currentMode == HUE) {
      currentMode = BRIGHTNESS;
      myEnc.write(brightnessPosition * 4); // Restore brightness encoder position
    } else {
      currentMode = HUE;
      myEnc.write(huePosition * 4); // Restore hue encoder position
    }
    Serial.print("Mode: ");
    Serial.println((currentMode == HUE) ? "HUE" : "BRIGHTNESS");
  }
  
  lastButtonState = buttonState;

  strip.show();
}

void updateLEDs() {
  for (int i = 0; i < NUM_LEDS; i++) {
    strip.setPixelColor(i, strip.ColorHSV(hue * 65536L / 256, 255, brightness));
  }
}
